@@ -9,11 +9,9 @@ import logging
 from itertools import islice
 import pandas as pd
 
-from celescope.tools.utils import format_number
+from celescope.tools.utils import format_number, log
 from celescope.tools.report import reporter
 
-logger1 = logging.getLogger(__name__)
-
 
 def format_stat(cutadapt_log, samplename):
     fh = open(cutadapt_log, 'r')
@@ -42,8 +40,8 @@ def format_stat(cutadapt_log, samplename):
     fh.close()
 
 
+@log
 def cutadapt(args):
-    logger1.info('cutadapt ...!')
     # check dir
     if not os.path.exists(args.outdir):
         os.system('mkdir -p %s' % (args.outdir))
@@ -67,11 +65,10 @@ def cutadapt(args):
                                   '-o',
                                   out_fq2,
                                   args.fq]
-    logger1.info('%s' % (' '.join(cmd)))
+    cutadapt.logger.info('%s' % (' '.join(cmd)))
     res = subprocess.run(cmd, stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
     with open(args.outdir + '/cutadapt.log', 'wb') as fh:
         fh.write(res.stdout)
-    logger1.info('cutadapt done!')
 
     format_stat(args.outdir + '/cutadapt.log', args.sample)
     t = reporter(
@@ -99,7 +96,7 @@ def get_opts_cutadapt(parser, sub_program):
             'p5=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC',
         ])
     parser.add_argument(
-        '--minimum-length',
+        '--minimum_length',
         dest='minimum_length',
         help='minimum_length, default=20',
         default=20)
@@ -110,6 +107,6 @@ def get_opts_cutadapt(parser, sub_program):
         default=20)
     parser.add_argument(
         '--overlap',
-        help='minimum overlap length, default=5',
-        default=5)
+        help='minimum overlap length',
+        default=10)
     parser.add_argument('--thread', default=2)
