@@ -9,10 +9,12 @@ import numpy as np
 import subprocess
 import time
 import argparse
+import pysam
 from datetime import timedelta
 from collections import defaultdict
 from functools import wraps
 from collections import Counter
+import json
 import celescope.tools
 import matplotlib
 matplotlib.use('Agg')
@@ -50,16 +52,25 @@ def log(func):
     return wrapper
 
 
+def arg_str(arg, arg_name):
+    '''
+    return action store_true arguments as string
+    '''
+    if arg:
+        return '--' + arg_name
+    return ''
+
+
 def read_barcode_file(match_dir):
     '''
     multi version compatible
     '''
     match_barcode_file1 = glob.glob(
-        f"{match_dir}/05.count/*_cellbarcode.tsv")
+        f"{match_dir}/*count/*_cellbarcode.tsv")
     match_barcode_file2 = glob.glob(
-        f"{match_dir}/05.count/*matrix_10X/*_cellbarcode.tsv")
+        f"{match_dir}/*count/*matrix_10X/*_cellbarcode.tsv")
     match_barcode_file3 = glob.glob(
-        f"{match_dir}/05.count/*matrix_10X/*barcodes.tsv")
+        f"{match_dir}/*count/*matrix_10X/*barcodes.tsv")
     match_barcode_file = (
         match_barcode_file1 +
         match_barcode_file2 +
@@ -71,7 +82,7 @@ def read_barcode_file(match_dir):
 
 def format_stat(count, total_count):
     percent = round(count / total_count * 100, 2)
-    string = f'{count}({percent}%)'
+    string = f'{format_number(count)}({percent}%)'
     return string
 
 
@@ -89,7 +100,7 @@ def multi_opts(assay):
             4th col: Cell number or match_dir, optional;
         ''',
         required=True)
-    parser.add_argument('--chemistry', choices=__PATTERN_DICT__.keys(), help='chemistry version')
+    parser.add_argument('--chemistry', choices=__PATTERN_DICT__.keys(), help='chemistry version', default='auto')
     parser.add_argument('--whitelist', help='cellbarcode list')
     parser.add_argument('--linker', help='linker')
     parser.add_argument('--pattern', help='read1 pattern')
@@ -102,7 +113,7 @@ def multi_opts(assay):
             'polyT=A{15}',
             'p5=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC'])
     parser.add_argument(
-        '--minimum-length',
+        '--minimum_length',
         dest='minimum_length',
         help='minimum_length',
         default=20)
@@ -129,6 +140,9 @@ def multi_opts(assay):
         '--rm_files',
         action='store_true',
         help='remove redundant fq.gz and bam after running')
+    parser.add_argument('--steps_run', help='steps to run', default='all')
+    parser.add_argument('--debug', help='debug or not', action='store_true')
+    parser.add_argument('--outFilterMatchNmin', help='STAR outFilterMatchNmin', default=0)
     return parser
 
 
@@ -158,7 +172,11 @@ def gene_convert(gtf_file):
             gtf_type, attributes = tabs[2], tabs[-1]
             if gtf_type == 'gene':
                 gene_id = gene_id_pattern.findall(attributes)[-1]
-                gene_name = gene_name_pattern.findall(attributes)[-1]
+                gene_names = gene_name_pattern.findall(attributes)
+                if not gene_names:
+                    gene_name = gene_id 
+                else:
+                    gene_name = gene_names[-1]
                 c[gene_name] += 1
                 if c[gene_name] > 1:
                     gene_name = f'{gene_name}_{c[gene_name]}'
@@ -167,7 +185,10 @@ def gene_convert(gtf_file):
 
 
 @log
-def process_read(read2_file, pattern_dict, barcode_dict, linker_dict):
+def process_read(
+    read2_file, pattern_dict, barcode_dict, linker_dict,
+    barcode_length, linker_length):
+
     # if valid, return (True)
     metrics = defaultdict(int)
     res_dict = genDict(dim=3)
@@ -184,26 +205,30 @@ def process_read(read2_file, pattern_dict, barcode_dict, linker_dict):
         barcode = str(attr[0])
         umi = str(attr[1])
         seq = line2.strip()
-        if linker_dict:
-            try:
-                seq_linker = ''.join(seq_range(seq, pattern_dict['L']))
-            except:
+        if linker_length != 0:
+            seq_linker = seq_ranges(seq, pattern_dict['L'])
+            if len(seq_linker) < linker_length:
                 metrics['Reads Unmapped too Short'] += 1
                 continue
         if barcode_dict:
-            try:
-                seq_barcode = ''.join(seq_range(seq, pattern_dict['C']))
-            except:
-                metrics['Reads Unmapped too Short'] += 1
-                continue
+            seq_barcode = seq_ranges(seq, pattern_dict['C'])
+            if barcode_length != len(seq_barcode):
+                miss_length = barcode_length - len(seq_barcode)
+                if miss_length > 2:
+                    metrics['Reads Unmapped too Short'] += 1
+                    continue
+                seq_barcode = seq_barcode + "A" * miss_length                    
         
         # check linker
-        valid_linker = False
-        for linker_name in linker_dict:
-            if hamming_correct(linker_dict[linker_name], seq_linker):
-                valid_linker = True
-                continue
-
+        if linker_length != 0:
+            valid_linker = False
+            for linker_name in linker_dict:
+                if hamming_correct(linker_dict[linker_name], seq_linker):
+                    valid_linker = True
+                    break
+        else:
+            valid_linker = True
+            
         if not valid_linker:
             metrics['Reads Unmapped Invalid Linker'] += 1
             continue
@@ -214,7 +239,7 @@ def process_read(read2_file, pattern_dict, barcode_dict, linker_dict):
             if hamming_correct(barcode_dict[barcode_name], seq_barcode):
                 res_dict[barcode][barcode_name][umi] += 1
                 valid_barcode = True
-                continue
+                break
 
         if not valid_barcode:
             metrics['Reads Unmapped Invalid Barcode'] += 1
@@ -228,7 +253,7 @@ def process_read(read2_file, pattern_dict, barcode_dict, linker_dict):
     return res_dict, metrics
 
 
-def seq_range(seq, pattern_dict):
+def seq_ranges_exception(seq, pattern_dict):
     # get subseq with intervals in arr and concatenate
     length = len(seq)
     for x in pattern_dict:
@@ -237,6 +262,11 @@ def seq_range(seq, pattern_dict):
     return ''.join([seq[x[0]:x[1]]for x in pattern_dict])
 
 
+def seq_ranges(seq, pattern_dict):
+    # get subseq with intervals in arr and concatenate
+    return ''.join([seq[x[0]:x[1]]for x in pattern_dict])
+
+
 def read_one_col(file):
     df = pd.read_csv(file, header=None)
     col1 = list(df.iloc[:, 0])
@@ -281,7 +311,7 @@ def hamming_distance(string1, string2):
     length = len(string1)
     length2 = len(string2)
     if (length != length2):
-        raise Exception("string1 and string2 do not have same length")
+        raise Exception(f"string1({length}) and string2({length2}) do not have same length")
     for i in range(length):
         if string1[i] != string2[i]:
             distance += 1
@@ -354,10 +384,12 @@ def parse_map_col4(mapfile, default_val):
         fq_dict[sample_name][0] = ",".join(fq_dict[sample_name][0])
         fq_dict[sample_name][1] = ",".join(fq_dict[sample_name][1])
 
+    if not fq_dict:
+        raise Exception('empty mapfile!')
     return fq_dict, col4_dict
 
 
-def generate_sjm(cmd, name, conda, m=1, x=1):
+def generate_sjm(cmd, name, conda, m=1, x=1, steps_run='all'):
     res_cmd = f'''
 job_begin
     name {name}
@@ -395,7 +427,7 @@ def format_number(number: int) -> str:
 
 
 @log
-def glob_genomeDir(genomeDir):
+def glob_genomeDir(genomeDir, fa=False):
     refFlat = glob.glob(genomeDir + "/*.refFlat")
     if (len(refFlat) > 1):
         sys.exit("ERROR: Multiple refFlat file in " + genomeDir)
@@ -420,7 +452,13 @@ def glob_genomeDir(genomeDir):
     else:
         gtf = gtf[0]
         glob_genomeDir.logger.info("gtf file found: " + gtf)
-
+    
+    if fa:
+        fasta = glob.glob(genomeDir + "/*.fa") + glob.glob(genomeDir + "/*.fasta")
+        if len(fasta) > 1:
+            sys.exit("ERROR: Multiple fasta file in " + genomeDir)
+        fasta = fasta[0]
+        return refFlat, gtf, fasta
     return refFlat, gtf
 
 
@@ -533,11 +571,11 @@ def barcode_filter_with_derivative(
     return (validated_barcodes, threshold, len(validated_barcodes))
 
 
-def genDict(dim=3):
+def genDict(dim=3, valType=int):
     if dim == 1:
-        return defaultdict(int)
+        return defaultdict(valType)
     else:
-        return defaultdict(lambda: genDict(dim - 1))
+        return defaultdict(lambda: genDict(dim - 1, valType=valType))
 
 
 @log
@@ -599,9 +637,201 @@ def downsample(bam, barcodes, percent):
         percent, median_geneNum, saturation), saturation
 
 
-if __name__ == '__main__':
+def cluster_tsne_list(tsne_df):
+    """
+    tSNE_1	tSNE_2	cluster Gene_Counts
+    return data list
+    """
+    sum_df = tsne_df.groupby(["cluster"]).agg("count").iloc[:, 0]
+    percent_df = sum_df.transform(lambda x: round(x / sum(x) * 100, 2))
+    res = []
+    for cluster in sorted(tsne_df.cluster.unique()):
+        sub_df = tsne_df[tsne_df.cluster == cluster]
+        name = "cluster {cluster}({percent}%)".format(
+            cluster=cluster, percent=percent_df[cluster])
+        tSNE_1 = list(sub_df.tSNE_1)
+        tSNE_2 = list(sub_df.tSNE_2)
+        res.append({"name": name, "tSNE_1": tSNE_1, "tSNE_2": tSNE_2})
+    return res
+
+
+def marker_table(marker_df):
+    """
+    return html code
+    """
+    marker_df = marker_df.loc[:, ["cluster", "gene",
+                                  "avg_logFC", "pct.1", "pct.2", "p_val_adj"]]
+    marker_gene_table = marker_df.to_html(
+        escape=False,
+        index=False,
+        table_id="marker_gene_table",
+        justify="center")
+    return marker_gene_table
+
+
+def report_prepare(outdir, **kwargs):
+    json_file = outdir + '/../.data.json'
+    if not os.path.exists(json_file):
+        data = {}
+    else:
+        fh = open(json_file)
+        data = json.load(fh)
+        fh.close()
+
+    for key in kwargs:
+        data[key] = kwargs[key]
+
+    with open(json_file, 'w') as fh:
+        json.dump(data, fh)
+
+
+def parse_vcf(vcf_file, cols=['chrom', 'pos', 'alleles'], infos=['DP', 'GENE', 'CELL']):
+    vcf = pysam.VariantFile(vcf_file)
+    df = pd.DataFrame(columns=[col.upper() for col in cols] + infos + ['GT'])
+    rec_dict = {}
+    for rec in vcf.fetch():
+
+        for col in cols:
+            rec_dict[col.upper()] = getattr(rec, col)
+            if col == 'alleles':
+                rec_dict['ALLELES'] = '-'.join(rec_dict['ALLELES'])
+                
+        for info in infos:
+            rec_dict[info] = rec.info[info]
+
+        rec_dict['GT'] = [s['GT'] for s in rec.samples.values()][0]
+        rec_dict['GT'] = [str(item) for item in rec_dict['GT']]
+        rec_dict['GT'] = '/'.join(rec_dict['GT'])
+
+        df = df.append(pd.Series(rec_dict),ignore_index=True)
+    return df
+
+
+def parse_annovar(annovar_file):
+    df = pd.DataFrame(columns=['mRNA', 'protein', 'cosmic'])
+    with open(annovar_file, 'rt') as f:
+        index = 0
+        for line in f:
+            index += 1
+            if index == 1:
+                continue
+            attrs = line.split('\t')
+            func = attrs[5]
+            if func == 'exonic':
+                changes = attrs[9]
+                cosmic = attrs[10]
+            else:
+                changes = attrs[7]
+                cosmic = attrs[8]
+            change_list = list()
+            for change in changes.split(','):
+                change_attrs = change.split(':')
+                mRNA = ''
+                protein = ''
+                for change_index in range(len(change_attrs)):
+                    change_attr = change_attrs[change_index]
+                    if change_attr.startswith('c.'):
+                        base = change_attr.strip('c.')
+                        exon = change_attrs[change_index - 1]
+                        mRNA = f'{exon}:{base}'
+                    if change_attr.startswith('p.'):
+                        protein = change_attr.strip('p.')
+                if not (mRNA, protein) in change_list:
+                    change_list.append((mRNA, protein)) 
+            combine = [','.join(item) for item in list(zip(*change_list))]
+            mRNA = combine[0]
+            protein = combine[1]
+            df = df.append({
+                'MRNA': mRNA,
+                'PROTEIN': protein,
+                'COSMIC': cosmic,
+            }, ignore_index=True)
+    return df
+
+
+def parse_match_dir(match_dir):
+    match_dict = {}
+    match_barcode, cell_total = read_barcode_file(match_dir)
+    match_dict['match_barcode'] = match_barcode
+    match_dict['cell_total'] = cell_total
+    match_dict['tsne_coord'] = glob.glob(f'{match_dir}/*analysis*/*tsne_coord.tsv')[0]
+    match_dict['markers'] = glob.glob(f'{match_dir}/*analysis*/*markers.tsv')[0]
+    try:
+        match_dict['rds'] = glob.glob(f'{match_dir}/*analysis/*.rds')[0]
+    except Exception:
+        match_dict['rds'] = None
+    return match_dict
+
+
+@log
+def STAR_util(
+    sample,
+    outdir,
+    input_read,
+    genomeDir,
+    runThreadN,
+    outFilterMatchNmin=35,
+    out_unmapped=False,
+    outFilterMultimapNmax=1,
+):
+
+    # check dir
+    if not os.path.exists(outdir):
+        os.system('mkdir -p %s' % (outdir))
+
+    out_prefix = f'{outdir}/{sample}_'
+    out_BAM = out_prefix + "Aligned.sortedByCoord.out.bam"
+
+    cmd = f"STAR \
+ --genomeDir {genomeDir} \
+ --readFilesIn {input_read}\
+ --readFilesCommand zcat\
+ --outSAMtype BAM SortedByCoordinate\
+ --runThreadN {runThreadN}\
+ --outFilterMatchNmin {outFilterMatchNmin}\
+ --outFileNamePrefix {out_prefix}\
+ --outFilterMultimapNmax {outFilterMultimapNmax} "
+    
+    if out_unmapped:
+        cmd += ' --outReadsUnmapped Fastx '
+
+    STAR_util.logger.info(cmd)
+    os.system(cmd)
+
+    cmd = "samtools index {out_BAM}".format(out_BAM=out_BAM)
+    STAR_util.logger.info(cmd)
+    os.system(cmd)
+
+
+def get_scope_bc(bctype):
+    import celescope
+    root_path = os.path.dirname(celescope.__file__)
+    linker_f = glob.glob(f'{root_path}/data/chemistry/{bctype}/linker*')[0]
+    whitelist_f = f'{root_path}/data/chemistry/{bctype}/bclist'
+    return linker_f, whitelist_f
+
+@log
+def parse_pattern(pattern):
+    # 解析接头结构，返回接头结构字典
+    # key: 字母表示的接头, value: 碱基区间列表
+    # eg.: C8L10C8L10C8U8T30
+    # defaultdict(<type 'list'>:
+    # {'C': [[0, 8], [18, 26], [36, 44]], 'U': [[44, 52]], 'L': [[8, 18], [26, 36]], 'T': [[52, 82]]})
+    pattern_dict = defaultdict(list)
+    p = re.compile(r'([CLUNT])(\d+)')
+    tmp = p.findall(pattern)
+    if not tmp:
+        parse_pattern.logger.error(f'Invalid pattern: {pattern}')
+        sys.exit()
+    start = 0
+    for item in tmp:
+        end = start + int(item[1])
+        pattern_dict[item[0]].append([start, end])
+        start = end
+    return pattern_dict
+    
+
+
+
+
 
-    df = pd.read_table('SRR6954578_counts.txt', header=0)
-    barcode_filter_with_magnitude(df)
-    barcode_filter_with_kde(df)
-    barcode_filter_with_derivative(df)
